var extend = require('util')._extend;
var postcss = require('postcss');
var mediaQuery = require('css-mediaquery');

var debug = false;

var defaultOpts = {
    // min: Number.MIN_VALUE,
    // max: 767,
    // removeMax: true,
    // removeMin: false

    min: Number.MIN_VALUE,
    max: Number.MAX_VALUE,
    removeMax: false,
    removeMin: false
};

module.exports = postcss.plugin('postcss-unmq', function (opts) {
    opts = extend(extend({}, defaultOpts), opts);

    return function (css) {
        css.walkAtRules('media', function (rule) {
            var mediaQueryAst = mediaQuery.parse(rule.params)[0];

            var min = Number.MIN_VALUE;
            var max = Number.MAX_VALUE;
            for (var i=0, expression; expression = mediaQueryAst.expressions[i]; i++) {
                if (expression.feature ==  'width') {
                    if (expression.modifier == "min") {
                        min = +expression.value.replace(/[a-zA-Z]/g, '');
                    } else if (expression.modifier == "max") {
                        max = +expression.value.replace(/[a-zA-Z]/g, '');
                    }
                }
            }

            /*remove rule if it is not in the defined range*/
            if ((min < opts.min && max < opts.min)
                || min > opts.max && max > opts.max) {
                if (debug) {
                    console.log('removed rule', rule.params);
                }

                rule.remove();
            } else {
                if (opts.removeMin && min === opts.min
                    || opts.removeMax && max === opts.max) {
                    var newMediaQueryParts = [];
                    if (mediaQueryAst.type) {
                        newMediaQueryParts.push(mediaQueryAst.type);
                    }

                    if (mediaQueryAst.expressions) {
                        mediaQueryAst.expressions.forEach(function (expression) {
                            if( (opts.removeMin && min === opts.min && expression.modifier == 'min')
                                || (opts.removeMax && max === opts.max && expression.modifier == 'max')) {
                                return;
                            }

                            var expressionString = '(';
                            if (expression.modifier) {
                                expressionString += expression.modifier + '-';
                            }
                            expressionString += expression.feature + ':';
                            expressionString += expression.value + ')';
                            newMediaQueryParts.push(expressionString);

                        });
                    }

                    var newMediaQuery = ' ' + newMediaQueryParts.join(' and ');
                    if (debug) {
                        console.log('replaced "', rule.params, '" with"', newMediaQuery, '"');
                    }
                    rule.params = newMediaQuery;
                }
            }
        });
    };
});
